@@ -2,7 +2,7 @@
     This class carries custom options
                              -------------------
     begin                : Fr 29 Apr 2011
-    copyright            : (C) 2011-2019 by Alexander Reinholdt
+    copyright            : (C) 2011-2021 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -23,10 +23,6 @@
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 // application specific includes
 #include "smb4kcustomoptions.h"
 #include "smb4ksettings.h"
@@ -53,7 +49,7 @@ class Smb4KCustomOptionsPrivate
     QUrl url;
     QHostAddress ip;
     NetworkItem type;
-    Smb4KCustomOptions::Remount remount;
+    int remount;
     bool useUser;
     KUser user;
     bool useGroup;
@@ -66,12 +62,17 @@ class Smb4KCustomOptionsPrivate
     bool cifsUnixExtensionsSupport;
     bool useFileSystemPort;
     int fileSystemPort;
+    bool useMountProtocolVersion;
+    int mountProtocolVersion;
     bool useSecurityMode;
     int securityMode;
     bool useWriteAccess;
     int writeAccess;
 #endif
     QString profile;
+    bool useClientProtocolVersions;
+    int minimalClientProtocolVersion;
+    int maximalClientProtocolVersion;
     bool useSmbPort;
     int smbPort;
     bool useKerberos;
@@ -89,9 +90,9 @@ Smb4KCustomOptions::Smb4KCustomOptions(Smb4KHost *host)
   d->type = Host;
   d->remount = UndefinedRemount;
   d->useUser = Smb4KMountSettings::useUserId();
-  d->user = KUser(Smb4KMountSettings::userId());
+  d->user = KUser((K_UID)Smb4KMountSettings::userId().toInt());
   d->useGroup = Smb4KMountSettings::useGroupId();
-  d->group = KUserGroup(Smb4KMountSettings::groupId());
+  d->group = KUserGroup((K_GID)Smb4KMountSettings::groupId().toInt());
   d->useFileMode = Smb4KMountSettings::useFileMode();
   d->fileMode = Smb4KMountSettings::fileMode();
   d->useDirectoryMode = Smb4KMountSettings::useDirectoryMode();
@@ -100,11 +101,16 @@ Smb4KCustomOptions::Smb4KCustomOptions(Smb4KHost *host)
   d->cifsUnixExtensionsSupport = Smb4KMountSettings::cifsUnixExtensionsSupport();
   d->useFileSystemPort = Smb4KMountSettings::useRemoteFileSystemPort();
   d->fileSystemPort = Smb4KMountSettings::remoteFileSystemPort();
+  d->useMountProtocolVersion = Smb4KMountSettings::useSmbProtocolVersion();
+  d->mountProtocolVersion = Smb4KMountSettings::smbProtocolVersion();
   d->useSecurityMode = Smb4KMountSettings::useSecurityMode();
   d->securityMode = Smb4KMountSettings::securityMode();
   d->useWriteAccess = Smb4KMountSettings::useWriteAccess();
   d->writeAccess = Smb4KMountSettings::writeAccess();
 #endif
+  d->useClientProtocolVersions = Smb4KSettings::useClientProtocolVersions();
+  d->minimalClientProtocolVersion = Smb4KSettings::minimalClientProtocolVersion();
+  d->maximalClientProtocolVersion = Smb4KSettings::maximalClientProtocolVersion();
   d->useSmbPort = Smb4KSettings::useRemoteSmbPort();
   d->smbPort = host->port() != -1 ? host->port() : Smb4KSettings::remoteSmbPort();
   d->useKerberos = Smb4KSettings::useKerberos();
@@ -132,11 +138,16 @@ Smb4KCustomOptions::Smb4KCustomOptions(Smb4KShare *share)
   d->cifsUnixExtensionsSupport = Smb4KMountSettings::cifsUnixExtensionsSupport();
   d->useFileSystemPort = Smb4KMountSettings::useRemoteFileSystemPort();
   d->fileSystemPort = share->port() != -1 ? share->port() : Smb4KMountSettings::remoteFileSystemPort();
+  d->useMountProtocolVersion = Smb4KMountSettings::useSmbProtocolVersion();
+  d->mountProtocolVersion = Smb4KMountSettings::smbProtocolVersion();
   d->useSecurityMode = Smb4KMountSettings::useSecurityMode();
   d->securityMode = Smb4KMountSettings::securityMode();
   d->useWriteAccess = Smb4KMountSettings::useWriteAccess();
   d->writeAccess = Smb4KMountSettings::writeAccess();
 #endif
+  d->useClientProtocolVersions = Smb4KSettings::useClientProtocolVersions();
+  d->minimalClientProtocolVersion = Smb4KSettings::minimalClientProtocolVersion();
+  d->maximalClientProtocolVersion = Smb4KSettings::maximalClientProtocolVersion();
   d->useSmbPort = Smb4KSettings::useRemoteSmbPort();
   d->smbPort = Smb4KSettings::remoteSmbPort();
   d->useKerberos = Smb4KSettings::useKerberos();
@@ -159,9 +170,9 @@ Smb4KCustomOptions::Smb4KCustomOptions()
   d->type = UnknownNetworkItem;
   d->remount = UndefinedRemount;
   d->useUser = Smb4KMountSettings::useUserId();
-  d->user = KUser(Smb4KMountSettings::userId());
+  d->user = KUser((K_UID)Smb4KMountSettings::userId().toInt());
   d->useGroup = Smb4KMountSettings::useGroupId();
-  d->group = KUserGroup(Smb4KMountSettings::groupId());
+  d->group = KUserGroup((K_GID)Smb4KMountSettings::groupId().toInt());
   d->useFileMode = Smb4KMountSettings::useFileMode();
   d->fileMode = Smb4KMountSettings::fileMode();
   d->useDirectoryMode = Smb4KMountSettings::useDirectoryMode();
@@ -170,11 +181,16 @@ Smb4KCustomOptions::Smb4KCustomOptions()
   d->cifsUnixExtensionsSupport = Smb4KMountSettings::cifsUnixExtensionsSupport();
   d->useFileSystemPort = Smb4KMountSettings::useRemoteFileSystemPort();
   d->fileSystemPort = Smb4KMountSettings::remoteFileSystemPort();
+  d->useMountProtocolVersion = Smb4KMountSettings::useSmbProtocolVersion();
+  d->mountProtocolVersion = Smb4KMountSettings::smbProtocolVersion();
   d->useSecurityMode = Smb4KMountSettings::useSecurityMode();
   d->securityMode = Smb4KMountSettings::securityMode();
   d->useWriteAccess = Smb4KMountSettings::useWriteAccess();
   d->writeAccess = Smb4KMountSettings::writeAccess();
 #endif
+  d->useClientProtocolVersions = Smb4KSettings::useClientProtocolVersions();
+  d->minimalClientProtocolVersion = Smb4KSettings::minimalClientProtocolVersion();
+  d->maximalClientProtocolVersion = Smb4KSettings::maximalClientProtocolVersion();
   d->useSmbPort = Smb4KSettings::useRemoteSmbPort();
   d->smbPort = Smb4KSettings::remoteSmbPort();
   d->useKerberos = Smb4KSettings::useKerberos();
@@ -354,7 +370,7 @@ QString Smb4KCustomOptions::displayString() const
 }
 
 
-void Smb4KCustomOptions::setRemount(Smb4KCustomOptions::Remount remount)
+void Smb4KCustomOptions::setRemount(int remount)
 {
   switch (d->type)
   {
@@ -372,7 +388,7 @@ void Smb4KCustomOptions::setRemount(Smb4KCustomOptions::Remount remount)
 }
 
 
-Smb4KCustomOptions::Remount Smb4KCustomOptions::remount() const
+int Smb4KCustomOptions::remount() const
 {
   return d->remount;
 }
@@ -524,6 +540,30 @@ int Smb4KCustomOptions::fileSystemPort() const
 }
 
 
+void Smb4KCustomOptions::setUseMountProtocolVersion(bool use)
+{
+  d->useMountProtocolVersion = use;
+}
+
+
+bool Smb4KCustomOptions::useMountProtocolVersion() const
+{
+  return d->useMountProtocolVersion;
+}
+
+
+void Smb4KCustomOptions::setMountProtocolVersion(int version)
+{
+  d->mountProtocolVersion = version;
+}
+
+
+int Smb4KCustomOptions::mountProtocolVersion() const
+{
+  return d->mountProtocolVersion;
+}
+
+
 void Smb4KCustomOptions::setUseSecurityMode(bool use)
 {
   d->useSecurityMode = use;
@@ -584,6 +624,41 @@ QString Smb4KCustomOptions::profile() const
   return d->profile;
 }
 
+void Smb4KCustomOptions::setUseClientProtocolVersions(bool use)
+{
+  d->useClientProtocolVersions = use;
+}
+
+
+bool Smb4KCustomOptions::useClientProtocolVersions() const
+{
+  return d->useClientProtocolVersions;
+}
+
+
+void Smb4KCustomOptions::setMinimalClientProtocolVersion(int version)
+{
+  d->minimalClientProtocolVersion = version;
+}
+
+
+int Smb4KCustomOptions::minimalClientProtocolVersion() const
+{
+  return d->minimalClientProtocolVersion;
+}
+
+
+void Smb4KCustomOptions::setMaximalClientProtocolVersion(int version)
+{
+  d->maximalClientProtocolVersion = version;
+}
+
+
+int Smb4KCustomOptions::maximalClientProtocolVersion() const
+{
+  return d->maximalClientProtocolVersion;
+}
+
 
 void Smb4KCustomOptions::setUseSmbPort(bool use)
 {
@@ -678,140 +753,110 @@ bool Smb4KCustomOptions::wolSendBeforeMount() const
 QMap<QString, QString> Smb4KCustomOptions::customOptions() const
 {
   QMap<QString,QString> entries;
-
-  switch (d->remount)
-  {
-    case RemountOnce:
-    {
-      entries.insert("remount", "once");
-      break;
-    }
-    case RemountAlways:
-    {
-      entries.insert("remount", "always");
-      break;
-    }
-    case UndefinedRemount:
-    {
-      entries.insert("remount", QString());
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
   
-  entries.insert("use_user", d->useUser ? "true" : "false");
-  entries.insert("uid", QString("%1").arg(d->user.userId().toString()));
-  entries.insert("owner", d->user.loginName());
-  entries.insert("use_group", d->useGroup ? "true" : "false");
-  entries.insert("gid", QString("%1").arg(d->group.groupId().toString()));
-  entries.insert("group", d->group.name());
-  entries.insert("use_file_mode", d->useFileMode ? "true" : "false");
+  //
+  // Remounting
+  // 
+  entries.insert("remount", QString("%1").arg(d->remount));
+  
+  //
+  // User
+  // 
+  entries.insert("use_user", QString("%1").arg(d->useUser));
+  entries.insert("uid", d->user.userId().toString());
+  
+  //
+  // Group
+  // 
+  entries.insert("use_group", QString("%1").arg(d->useGroup));
+  entries.insert("gid", d->group.groupId().toString());
+  
+  //
+  // File mode
+  // 
+  entries.insert("use_file_mode", QString("%1").arg(d->useFileMode));
   entries.insert("file_mode", d->fileMode);
-  entries.insert("use_directory_mode", d->useDirectoryMode ? "true" : "false");
+  
+  //
+  // Directory mode
+  // 
+  entries.insert("use_directory_mode", QString("%1").arg(d->useDirectoryMode));
   entries.insert("directory_mode", d->directoryMode);
   
 #if defined(Q_OS_LINUX)
-  entries.insert("cifs_unix_extensions_support", d->cifsUnixExtensionsSupport ? "true" : "false");
-  entries.insert("use_filesystem_port", d->useFileSystemPort ? "true" : "false");
+  //
+  // Unix CIFS extensions supported
+  // 
+  entries.insert("cifs_unix_extensions_support", QString("%1").arg(d->cifsUnixExtensionsSupport));
+  
+  //
+  // File system port
+  // 
+  entries.insert("use_filesystem_port", QString("%1").arg(d->useFileSystemPort));
   entries.insert("filesystem_port", QString("%1").arg(fileSystemPort()));
-  entries.insert("use_security_mode", d->useSecurityMode ? "true" : "false");
   
-  switch (d->securityMode)
-  {
-    case Smb4KMountSettings::EnumSecurityMode::None:
-    {
-      entries.insert("security_mode", "none");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Krb5:
-    {
-      entries.insert("security_mode", "krb5");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Krb5i:
-    {
-      entries.insert("security_mode", "krb5i");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Ntlm:
-    {
-      entries.insert("security_mode", "ntlm");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Ntlmi:
-    {
-      entries.insert("security_mode", "ntlmi");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Ntlmv2:
-    {
-      entries.insert("security_mode", "ntlmv2");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Ntlmv2i:
-    {
-      entries.insert("security_mode", "ntlmv2i");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Ntlmssp:
-    {
-      entries.insert("security_mode", "ntlmssp");
-      break;
-    }
-    case Smb4KMountSettings::EnumSecurityMode::Ntlmsspi:
-    {
-      entries.insert("security_mode", "ntlmsspi");
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
+  // 
+  // Mount protocol version
+  // 
+  entries.insert("use_smb_mount_protocol_version", QString("%1").arg(d->useMountProtocolVersion));
+  entries.insert("smb_mount_protocol_version", QString("%1").arg(d->mountProtocolVersion));
   
-  entries.insert("use_write_access", d->useWriteAccess ? "true" : "false");
-
-  switch (d->writeAccess)
-  {
-    case Smb4KMountSettings::EnumWriteAccess::ReadWrite:
-    {
-      entries.insert("write_access", "true");
-      break;
-    }
-    case Smb4KMountSettings::EnumWriteAccess::ReadOnly:
-    {
-      entries.insert("write_access", "false");
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
+  //
+  // Security mode
+  // 
+  entries.insert("use_security_mode", QString("%1").arg(d->useSecurityMode));
+  entries.insert("security_mode", QString("%1").arg(d->securityMode));
+  
+  //
+  // Write access
+  // 
+  entries.insert("use_write_access", QString("%1").arg(d->useWriteAccess));
+  entries.insert("write_access", QString("%1").arg(d->writeAccess));
 #endif
-  entries.insert("use_smb_port", d->useSmbPort ? "true" : "false");
+ 
+  //
+  // Client protocol versions
+  // 
+  entries.insert("use_client_protocol_versions", QString("%1").arg(d->useClientProtocolVersions));
+  entries.insert("minimal_client_protocol_version", QString("%1").arg(d->minimalClientProtocolVersion));
+  entries.insert("maximal_client_protocol_version", QString("%1").arg(d->maximalClientProtocolVersion));
+  
+  // 
+  // SMB port used by the client 
+  // 
+  entries.insert("use_smb_port", QString("%1").arg(d->useSmbPort));
   entries.insert("smb_port", QString("%1").arg(smbPort()));
-  entries.insert("kerberos", d->useKerberos ? "true" : "false");
+  
+  // 
+  // Usage of Kerberos
+  // 
+  entries.insert("kerberos", QString("%1").arg(d->useKerberos));
+  
+  //
+  // MAC address
+  // 
   entries.insert("mac_address", d->mac);
-  entries.insert("wol_send_before_first_scan", d->wakeOnLanBeforeFirstScan ? "true" : "false");
-  entries.insert("wol_send_before_mount", d->wakeOnLanBeforeMount ? "true" : "false");
+  
+  //
+  // Wake-On_LAN settings
+  // 
+  entries.insert("wol_send_before_first_scan", QString("%1").arg(d->wakeOnLanBeforeFirstScan));
+  entries.insert("wol_send_before_mount", QString("%1").arg(d->wakeOnLanBeforeMount));
   
   return entries;
 }
 
 
-bool Smb4KCustomOptions::hasOptions() const
+bool Smb4KCustomOptions::hasOptions(bool withoutRemountOnce) const
 {
   //
   // NOTE: This function does not honor the workgroup, the url,
   // the ip address, the type and the profile, because these things 
   // are not custom options.
   // 
-  // Perform remount
-  if (d->remount != Smb4KCustomOptions::UndefinedRemount)
+  
+  // Perform remounts
+  if ((!withoutRemountOnce && d->remount != Smb4KCustomOptions::UndefinedRemount) || d->remount == Smb4KCustomOptions::RemountAlways)
   {
     return true;
   }
@@ -879,6 +924,18 @@ bool Smb4KCustomOptions::hasOptions() const
     return true;
   }
   
+  // Use SMB mount protocol version
+  if (d->useMountProtocolVersion != Smb4KMountSettings::useSmbProtocolVersion())
+  {
+    return true;
+  }
+  
+  // SMB mount protocol version
+  if (d->mountProtocolVersion != Smb4KMountSettings::smbProtocolVersion())
+  {
+    return true;
+  }
+  
   // Use security mode
   if (d->useSecurityMode != Smb4KMountSettings::useSecurityMode())
   {
@@ -904,6 +961,24 @@ bool Smb4KCustomOptions::hasOptions() const
   }
 #endif
 
+  // Use client protocol versions
+  if (d->useClientProtocolVersions != Smb4KSettings::useClientProtocolVersions())
+  {
+    return true;
+  }
+  
+  // Minimal client protocol version
+  if (d->minimalClientProtocolVersion != Smb4KSettings::minimalClientProtocolVersion())
+  {
+    return true;
+  }
+  
+  // Maximal client protocol version
+  if (d->maximalClientProtocolVersion != Smb4KSettings::maximalClientProtocolVersion())
+  {
+    return true;
+  }
+
   // Use SMB port
   if (d->useSmbPort != Smb4KSettings::useRemoteSmbPort())
   {
@@ -961,11 +1036,16 @@ void Smb4KCustomOptions::update(Smb4KCustomOptions *options)
     d->cifsUnixExtensionsSupport = options->cifsUnixExtensionsSupport();
     d->useFileSystemPort = options->useFileSystemPort();
     d->fileSystemPort = options->fileSystemPort();
+    d->useMountProtocolVersion = options->useMountProtocolVersion();
+    d->mountProtocolVersion = options->mountProtocolVersion();
     d->useSecurityMode = options->useSecurityMode();
     d->securityMode = options->securityMode();
     d->useWriteAccess = options->useWriteAccess();
     d->writeAccess = options->writeAccess();
 #endif
+    d->useClientProtocolVersions = options->useClientProtocolVersions();
+    d->minimalClientProtocolVersion = options->minimalClientProtocolVersion();
+    d->maximalClientProtocolVersion = options->maximalClientProtocolVersion();
     d->profile = options->profile();
     d->useSmbPort = options->useSmbPort();
     d->smbPort = options->smbPort();
