@@ -2,7 +2,7 @@
     This class carries custom options
                              -------------------
     begin                : Fr 29 Apr 2011
-    copyright            : (C) 2011-2019 by Alexander Reinholdt
+    copyright            : (C) 2011-2021 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -184,7 +184,6 @@ class Q_DECL_EXPORT Smb4KCustomOptions
      *                          is started.
      * @param RemountAlways     Remount the share every time the application is
      *                          started.
-     * @param RemountNever      Never remount the share.
      * @param UndefinedRemount  No remount behavior is undefined.
      */
     enum Remount { RemountOnce,
@@ -197,14 +196,14 @@ class Q_DECL_EXPORT Smb4KCustomOptions
      * 
      * @param remount       One entry of the Remount enumeration
      */
-    void setRemount(Remount remount);
+    void setRemount(int remount);
     
     /**
      * Returns if the network item should be remounted.
      * 
      * @returns if the network item should be remounted.
      */
-    Remount remount() const;
+    int remount() const;
     
     /**
      * Set if the information about the user that is to be owner of the share 
@@ -365,6 +364,35 @@ class Q_DECL_EXPORT Smb4KCustomOptions
      * @returns the file system port
      */
     int fileSystemPort() const;
+
+    /**
+     * Set if the SMB protocol version for mounting should be set.
+     * 
+     * @param use             Boolean that determines if the SMB protocol version
+     *                        for mounting should be set
+     */
+    void setUseMountProtocolVersion(bool use);
+    
+    /**
+     * Returns if the SMB protocol version for mounting should be set.
+     * 
+     * @returns TRUE if the SMB protocol version for mounting should be set.
+     */
+    bool useMountProtocolVersion() const;
+    
+    /**
+     * Set the SMB protocol version for mounting.
+     * 
+     * @param version         The protocol version used for mounting
+     */
+    void setMountProtocolVersion(int version);
+    
+    /**
+     * Returns the SMB protocol version for mounting.
+     * 
+     * @returns the SMB protocol version
+     */
+    int mountProtocolVersion() const;
     
     /**
      * Set if the security mode should be used.
@@ -441,6 +469,50 @@ class Q_DECL_EXPORT Smb4KCustomOptions
      */
     QString profile() const;
     
+    /**
+     * Set whether the client minimal and maximal protocol versions should
+     * be set/used.
+     * 
+     * @param use             TRUE, if the protocol versions should used
+     */
+    void setUseClientProtocolVersions(bool use);
+    
+    /**
+     * Returns whether the client minimal and maximal protocol versions
+     * should be set/used.
+     * 
+     * @returns TRUE if the client protocol versions should be set/used.
+     */
+    bool useClientProtocolVersions() const;
+    
+    /**
+     * Set the minimal client protocol version to use.
+     * 
+     * @param version         The SMB protocol version
+     */
+    void setMinimalClientProtocolVersion(int version);
+    
+    /**
+     * Return the minimal client protocol version to use.
+     * 
+     * @returns the minimal client protocol version to use.
+     */
+    int minimalClientProtocolVersion() const;
+    
+    /**
+     * Set the maximal client protocol version to use.
+     * 
+     * @param version         The SMB protocol version
+     */
+    void setMaximalClientProtocolVersion(int version);
+    
+    /**
+     * Return the maximal client protocol version to use.
+     * 
+     * @returns the maximal client protocol version to use.
+     */
+    int maximalClientProtocolVersion() const;
+    
     /**
      * Set whether the SMB port should be used.
      * 
@@ -552,11 +624,12 @@ class Q_DECL_EXPORT Smb4KCustomOptions
     QMap<QString,QString> customOptions() const;
     
     /**
-     * Check if there are options defined
+     * Check if there are options defined. If @p withoutRemountOnce is set, 
+     * this function will ignore the setting Smb4KCustomOptions::RemountOnce.
      * 
      * @returns TRUE if there are options defined and FALSE otherwise
      */
-    bool hasOptions() const;
+    bool hasOptions(bool withoutRemountOnce = false) const;
     
     /**
      * Update this custom options object. You cannot change the workgroup,
