@@ -2,7 +2,7 @@
     Manage custom options
                              -------------------
     begin                : Fr 29 Apr 2011
-    copyright            : (C) 2011-2019 by Alexander Reinholdt
+    copyright            : (C) 2011-2021 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -23,10 +23,6 @@
  *   MA 02110-1335, USA                                                    *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 // application specific includes
 #include "smb4kcustomoptionsmanager.h"
 #include "smb4kcustomoptionsmanager_p.h"
@@ -171,23 +167,23 @@ void Smb4KCustomOptionsManager::clearRemounts(bool force)
   // Remove the remount flag and, if there are nomore options defined,
   // also the options object. Write everything to the file afterwards.
   //
-  for (const OptionsPtr &o : d->options)
+  for (const OptionsPtr &options : d->options)
   {
-    if (o->type() == Share)
+    if (options->type() == Share)
     {
-      if (o->remount() == Smb4KCustomOptions::RemountOnce)
+      if (options->remount() == Smb4KCustomOptions::RemountOnce)
       {
-        o->setRemount(Smb4KCustomOptions::UndefinedRemount);
+        options->setRemount(Smb4KCustomOptions::UndefinedRemount);
       }
-      else if (o->remount() == Smb4KCustomOptions::RemountAlways && force)
+      else if (options->remount() == Smb4KCustomOptions::RemountAlways && force)
       {
-        o->setRemount(Smb4KCustomOptions::UndefinedRemount);
+        options->setRemount(Smb4KCustomOptions::UndefinedRemount);
       }
     }
     
-    if (!o->hasOptions())
+    if (!options->hasOptions())
     {
-      removeCustomOptions(o, false);
+      removeCustomOptions(options, false);
     }
   }
   
@@ -203,7 +199,7 @@ QList<OptionsPtr> Smb4KCustomOptionsManager::sharesToRemount()
   //
   // List of relevant custom options
   //
-  QList<OptionsPtr> options = customOptions(false);
+  QList<OptionsPtr> optionsList = customOptions(false);
   
   //
   // List of remounts
@@ -213,15 +209,11 @@ QList<OptionsPtr> Smb4KCustomOptionsManager::sharesToRemount()
   //
   // Get the list of remounts
   //
-  for (const OptionsPtr &o : options)
+  for (const OptionsPtr &options : optionsList)
   {
-    if (o->remount() == Smb4KCustomOptions::RemountOnce)
+    if (options->remount() != Smb4KCustomOptions::UndefinedRemount)
     {
-      remounts << o;
-    }
-    else if (o->remount() == Smb4KCustomOptions::RemountAlways)
-    {
-      remounts << o;
+      remounts << options;
     }
   }
   
@@ -380,13 +372,17 @@ void Smb4KCustomOptionsManager::readCustomOptions()
       if (xmlReader.isStartElement())
       {
         if (xmlReader.name() == "custom_options" && 
-            (xmlReader.attributes().value("version") != "1.2" && xmlReader.attributes().value("version") != "2.0"))
+            (xmlReader.attributes().value("version") != "2.0" && xmlReader.attributes().value("version") != "3.0"))
         {
           xmlReader.raiseError(i18n("The format of %1 is not supported.", xmlFile.fileName()));
           break;
         }
-        else
+        else if (xmlReader.name() == "custom_options" && xmlReader.attributes().value("version") == "2.0")
         {
+          // 
+          // NOTE: This is for backward compatibility only (since version 3.0.73) and 
+          // should be removed in the future again.
+          // 
           if (xmlReader.name() == "options")
           {
             OptionsPtr options = OptionsPtr(new Smb4KCustomOptions());
@@ -419,12 +415,6 @@ void Smb4KCustomOptionsManager::readCustomOptions()
                   QUrl url(xmlReader.readElementText());
                   options->setUrl(url);
                 }
-                else if (xmlReader.name() == "unc")
-                {
-                  QUrl url = QUrl::fromUserInput(xmlReader.readElementText());
-                  url.setScheme("smb");
-                  options->setUrl(url);
-                }
                 else if (xmlReader.name() == "ip")
                 {
                   options->setIpAddress(xmlReader.readElementText());
@@ -737,6 +727,338 @@ void Smb4KCustomOptionsManager::readCustomOptions()
               }
             }            
               
+            d->options << options;  
+          }
+        }
+        else
+        {
+          if (xmlReader.name() == "options")
+          {
+            OptionsPtr options = OptionsPtr(new Smb4KCustomOptions());
+            options->setProfile(xmlReader.attributes().value("profile").toString());
+            
+            //
+            // Initialize the options 
+            // 
+            if (QString::compare(xmlReader.attributes().value("type").toString(), "host", Qt::CaseInsensitive) == 0)
+            {
+              options->setHost(new Smb4KHost());
+            }
+            else
+            {
+              options->setShare(new Smb4KShare());
+            }
+
+            while (!(xmlReader.isEndElement() && xmlReader.name() == "options"))
+            {
+              xmlReader.readNext();
+  
+              if (xmlReader.isStartElement())
+              {
+                if (xmlReader.name() == "workgroup")
+                {
+                  options->setWorkgroupName(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "url")
+                {
+                  QUrl url(xmlReader.readElementText());
+                  options->setUrl(url);
+                }
+                else if (xmlReader.name() == "ip")
+                {
+                  options->setIpAddress(xmlReader.readElementText());
+                }
+                else if (xmlReader.name() == "custom")
+                {
+                  while (!(xmlReader.isEndElement() && xmlReader.name() == "custom"))
+                  {
+                    xmlReader.readNext();
+                    
+                    if (xmlReader.isStartElement())
+                    {
+                      if (xmlReader.name() == "smb_port")
+                      {
+                        bool ok = false;
+                        int portNumber = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setSmbPort(portNumber);
+                        }
+                      }
+                      else if (xmlReader.name() == "use_smb_port")
+                      {
+                        bool ok = false;
+                        bool useSmbPort = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseSmbPort(useSmbPort);
+                        }
+                      }
+                      else if (xmlReader.name() == "kerberos")
+                      {
+                        bool ok = false;
+                        bool useKerberos = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseKerberos(useKerberos);
+                        }
+                      }
+                      else if (xmlReader.name() == "mac_address")
+                      {
+                        QString macAddress = xmlReader.readElementText();
+                        
+                        QRegExp exp("..\\:..\\:..\\:..\\:..\\:..");
+  
+                        if (exp.exactMatch(macAddress))
+                        {
+                          options->setMACAddress(macAddress);
+                        }
+                      }
+                      else if (xmlReader.name() == "wol_send_before_first_scan")
+                      {
+                        bool ok = false;
+                        bool send = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setWOLSendBeforeNetworkScan(send);
+                        }
+                      }
+                      else if (xmlReader.name() == "wol_send_before_mount")
+                      {
+                        bool ok = false;
+                        bool send = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setWOLSendBeforeMount(send);
+                        }
+                      }
+                      else if (xmlReader.name() == "remount")
+                      {
+                        bool ok = false;
+                        int remount = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setRemount(remount);
+                        }
+                      }
+                      else if (xmlReader.name() == "use_user")
+                      {
+                        bool ok = false;
+                        bool useUser = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseUser(useUser);
+                        }
+                      }
+                      else if (xmlReader.name() == "uid")
+                      {
+                        bool ok = false;
+                        int uid = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          KUser user((K_UID)uid);
+                        
+                          if (user.isValid())
+                          {
+                            options->setUser(user);
+                          }
+                        }
+                      }
+                      else if (xmlReader.name() == "use_group")
+                      {
+                        bool ok = false;
+                        bool useGroup = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseGroup(useGroup);
+                        }
+                      }
+                      else if (xmlReader.name() == "gid")
+                      {
+                        bool ok = false;
+                        int gid = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          KUserGroup group((K_GID)gid);
+                        
+                          if (group.isValid())
+                          {
+                            options->setGroup(group);
+                          }
+                        }
+                      }
+                      else if (xmlReader.name() == "use_file_mode")
+                      {
+                        bool ok = false;
+                        bool useFileMode = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseFileMode(useFileMode);
+                        }
+                      }
+                      else if (xmlReader.name() == "file_mode")
+                      {
+                        options->setFileMode(xmlReader.readElementText());
+                      }
+                      else if (xmlReader.name() == "use_directory_mode")
+                      {
+                        bool ok = false;
+                        bool useDirectoryMode = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseDirectoryMode(useDirectoryMode);
+                        }
+                      }
+                      else if (xmlReader.name() == "directory_mode")
+                      {
+                        options->setDirectoryMode(xmlReader.readElementText());
+                      }
+                      else if (xmlReader.name() == "use_client_protocol_versions")
+                      {
+                        bool ok = false;
+                        bool useClientProtocolVersions = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseClientProtocolVersions(useClientProtocolVersions);
+                        }
+                      }
+                      else if (xmlReader.name() == "minimal_client_protocol_version")
+                      {
+                        bool ok = false;
+                        int minimalClientProtocolVersion = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setMinimalClientProtocolVersion(minimalClientProtocolVersion);
+                        }
+                      }
+                      else if (xmlReader.name() == "maximal_client_protocol_version")
+                      {
+                        bool ok = false;
+                        int maximalClientProtocolVersion = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setMaximalClientProtocolVersion(maximalClientProtocolVersion);
+                        }
+                      }
+#if defined(Q_OS_LINUX)
+                      else if (xmlReader.name() == "cifs_unix_extensions_support")
+                      {
+                        bool ok = false;
+                        bool cifsUnixExtensionsSupported = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setCifsUnixExtensionsSupport(cifsUnixExtensionsSupported);
+                        }
+                      }
+                      else if (xmlReader.name() == "use_filesystem_port")
+                      {
+                        bool ok = false;
+                        bool useFilesystemPort = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseFileSystemPort(useFilesystemPort);
+                        }
+                      }
+                      else if (xmlReader.name() == "filesystem_port")
+                      {
+                        bool ok = false;
+                        int portNumber = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setFileSystemPort(portNumber);
+                        }
+                      }
+                      else if (xmlReader.name() == "use_smb_mount_protocol_version")
+                      {
+                        bool ok = false;
+                        bool useMountProtocolVersion = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseMountProtocolVersion(useMountProtocolVersion);
+                        }
+                      }
+                      else if (xmlReader.name() =="smb_mount_protocol_version")
+                      {
+                        bool ok = false;
+                        int mountProtocolVersion = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setMountProtocolVersion(mountProtocolVersion);
+                        }
+                      }
+                      else if (xmlReader.name() == "use_security_mode")
+                      {
+                        bool ok = false;
+                        bool useSecurityMode = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseSecurityMode(useSecurityMode);
+                        }
+                      }
+                      else if (xmlReader.name() == "security_mode")
+                      {
+                        bool ok = false;
+                        int securityMode = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setSecurityMode(securityMode);
+                        }
+                      }
+                      else if (xmlReader.name() == "use_write_access")
+                      {
+                        bool ok = false;
+                        bool useWriteAccess = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setUseWriteAccess(useWriteAccess);
+                        }
+                      }
+                      else if (xmlReader.name() == "write_access")
+                      {
+                        bool ok = false;
+                        int writeAccess = xmlReader.readElementText().toInt(&ok);
+                        
+                        if (ok)
+                        {
+                          options->setWriteAccess(writeAccess);
+                        }
+                      }
+#endif
+                    }
+                  }
+                }
+  
+                continue;
+              }
+              else
+              {
+                continue;
+              }
+            }            
+              
             d->options << options;  
           }
         }
@@ -778,11 +1100,11 @@ void Smb4KCustomOptionsManager::writeCustomOptions()
       xmlWriter.setAutoFormatting(true);
       xmlWriter.writeStartDocument();
       xmlWriter.writeStartElement("custom_options");
-      xmlWriter.writeAttribute("version", "2.0");
+      xmlWriter.writeAttribute("version", "3.0");
       
       for (const OptionsPtr &options : d->options)
       {
-        if (options->hasOptions() || options->remount() == Smb4KCustomOptions::RemountOnce)
+        if (options->hasOptions())
         {
           xmlWriter.writeStartElement("options");
           xmlWriter.writeAttribute("type", options->type() == Host ? "host" : "share");
@@ -827,75 +1149,33 @@ void Smb4KCustomOptionsManager::writeCustomOptions()
 }
 
 
-QList<OptionsPtr> Smb4KCustomOptionsManager::customOptions(bool optionsOnly)
+QList<OptionsPtr> Smb4KCustomOptionsManager::customOptions(bool withoutRemountOnce)
 {
   //
   // Options list
   //
-  QList<OptionsPtr> options;
+  QList<OptionsPtr> optionsList;
 
   //
   // Get this list of options
   //
-  for (const OptionsPtr &o : d->options)
+  for (const OptionsPtr &options : d->options)
   {
-    if (Smb4KSettings::useProfiles() && o->profile() != Smb4KProfileManager::self()->activeProfile())
+    if (Smb4KSettings::useProfiles() && options->profile() != Smb4KProfileManager::self()->activeProfile())
     {
       continue;
     }
 
-    if (o->hasOptions() || (!optionsOnly && o->remount() == Smb4KCustomOptions::RemountOnce))
+    if (options->hasOptions(withoutRemountOnce))
     {
-      options << o;
+      optionsList << options;
     }
   }
   
   //
   // Return the list of relevant options
   //
-  return options;
-}
-
-
-void Smb4KCustomOptionsManager::replaceCustomOptions(const QList<OptionsPtr> &optionsList)
-{
-  //
-  // Clear the list of options. Honor profiles.
-  //
-  QMutableListIterator<OptionsPtr> it(d->options);
-  
-  while (it.hasNext())
-  {
-    OptionsPtr options = it.next();
-    
-    if (Smb4KSettings::useProfiles() && options->profile() != Smb4KProfileManager::self()->activeProfile())
-    {
-      continue;
-    }
-    
-    it.remove();
-  }
-  
-  //
-  // Append the new list
-  //
-  if (!optionsList.isEmpty())
-  {
-    for (const OptionsPtr &options : optionsList)
-    {
-      if (Smb4KSettings::useProfiles())
-      {
-        options->setProfile(Smb4KProfileManager::self()->activeProfile());
-      }
-      
-      if (options->hasOptions() || options->remount() == Smb4KCustomOptions::RemountOnce)
-      {
-        d->options << options;
-      }
-    }
-  }
-  
-  writeCustomOptions();
+  return optionsList;
 }
 
 
@@ -1047,6 +1327,8 @@ void Smb4KCustomOptionsManager::addCustomOptions(const OptionsPtr &options, bool
           o->setCifsUnixExtensionsSupport(options->cifsUnixExtensionsSupport());
           o->setUseFileSystemPort(options->useFileSystemPort());
           o->setFileSystemPort(options->fileSystemPort());
+          o->setUseMountProtocolVersion(options->useMountProtocolVersion());
+          o->setMountProtocolVersion(options->mountProtocolVersion());
           o->setUseSecurityMode(options->useSecurityMode());
           o->setSecurityMode(options->securityMode());
           o->setUseWriteAccess(options->useWriteAccess());
@@ -1083,8 +1365,7 @@ void Smb4KCustomOptionsManager::removeCustomOptions(const OptionsPtr &options, b
     for (int i = 0; i < d->options.size(); ++i)
     {
       if ((!Smb4KSettings::useProfiles() || Smb4KProfileManager::self()->activeProfile() == d->options.at(i)->profile()) &&
-          d->options.at(i)->url().toString(QUrl::RemoveUserInfo|QUrl::RemovePort|QUrl::StripTrailingSlash) == 
-          options->url().toString(QUrl::RemoveUserInfo|QUrl::RemovePort|QUrl::StripTrailingSlash))
+          d->options.at(i)->url().matches(options->url(), QUrl::RemoveUserInfo|QUrl::RemovePort|QUrl::StripTrailingSlash))
       {
         d->options.takeAt(i).clear();
         break;
@@ -1104,7 +1385,7 @@ void Smb4KCustomOptionsManager::removeCustomOptions(const OptionsPtr &options, b
 
 QList<OptionsPtr> Smb4KCustomOptionsManager::wakeOnLanEntries() const
 {
-  QList<OptionsPtr> list;
+  QList<OptionsPtr> optionsList;
   
   //
   // Get the Wake-On-LAN entries
@@ -1113,14 +1394,14 @@ QList<OptionsPtr> Smb4KCustomOptionsManager::wakeOnLanEntries() const
   {
     if (!options->macAddress().isEmpty() && (options->wolSendBeforeNetworkScan() || options->wolSendBeforeMount()))
     {
-      list << options;
+      optionsList << options;
     }
   }
   
   // 
   // Return them
   //
-  return list;
+  return optionsList;
 }
 
 
@@ -1130,6 +1411,12 @@ void Smb4KCustomOptionsManager::resetCustomOptions()
 }
 
 
+void Smb4KCustomOptionsManager::saveCustomOptions()
+{
+  writeCustomOptions();
+}
+
+
 void Smb4KCustomOptionsManager::migrateProfile(const QString& from, const QString& to)
 {
   //
